<?php
namespace kwy\wechatapi\openwx\common;

/**
 * XmlParse class
 *
 * 提供提取消息格式中的密文及生成回复消息格式的接口.
 */
class XmlParse
{
    /**
     * 提取出xml数据包中的加密消息
     * @author roc
     * @datetime:2019/12/17 14:46
     * @param string $xmlText 待提取的xml字符串
     * @return array 提取出的加密消息字符串
     * @throws \Exception
     */
	public function extract($xmlText)
	{
		try {
			$xml = new \DOMDocument();
			$xml->loadXML($xmlText);
			$array_e = $xml->getElementsByTagName('Encrypt');
			$array_a = $xml->getElementsByTagName('ToUserName');
			$encrypt = $array_e->item(0)->nodeValue;

			if($array_a->length > 0){
                $toUserName = $array_a->item(0)->nodeValue;
            }else{
                $toUserName = "";
            }
			return [$encrypt, $toUserName];
		} catch (\Exception $e) {
            throw new \Exception("xml解析失败");
		}
	}

    /**
     * 生成xml消息
     * @author roc
     * @datetime:2019/12/23 14:19
     * @param string $encrypt 加密后的消息密文
     * @param string $signature 安全签名
     * @param string $timestamp 时间戳
     * @param string $nonce 随机字符串
     * @return string
     */
	public function generate($encrypt, $signature, $timestamp, $nonce)
	{
		$format = "<xml>
<Encrypt><![CDATA[%s]]></Encrypt>
<MsgSignature><![CDATA[%s]]></MsgSignature>
<TimeStamp>%s</TimeStamp>
<Nonce><![CDATA[%s]]></Nonce>
</xml>";
		return sprintf($format, $encrypt, $signature, $timestamp, $nonce);
	}

}


?>