<?php
/**
 *
 * Auther: heyu
 * DateTime: 2020-01-03 15:01
 *
 */

namespace kwy\wechatapi\pay;


use kwy\util\http\CurlHttp;

class WechatPay extends ApiRequest
{

    private $mch_id='';

    private $appid='';

    private $sub_mch_id='';

    private $key='';

    /**
     * @param string $appid
     */
    public function setAppid($appid)
    {
        $this->appid = $appid;
    }

    /**
     * @param string $mch_id
     */
    public function setMchId($mch_id)
    {
        $this->mch_id = $mch_id;
    }

    /**
     * @param string $key
     */
    public function setKey($key)
    {
        $this->key = $key;
    }

    /**
     * @param string $sub_mch_id
     */
    public function setSubMchId(string $sub_mch_id)
    {
        $this->sub_mch_id = $sub_mch_id;
    }

    /**
     * 获取刷脸付设备授权信息
     * @author heyu 2020-01-19 11:13
     * @param string $store_id 门店编号， 由商户定义， 各门店唯一
     * @param string $store_name 门店名称，由商户定义。（可用于展示）
     * @param string $device_id 终端设备编号，由商户定义。
     * @param string $rawdata 设备初始化信息
     * @return array
     * @example [
     *  'authinfo'=>'sdasdasd',
     * 'expires_in'>7200
     * ]
     */
    public function facePayAuthInfo($store_id,$store_name,$device_id,$rawdata){
        $url='https://payapp.weixin.qq.com/face/get_wxpayface_authinfo';
        $params=[
            'store_id'=>$store_id,
            'store_name'=>$store_name,
            'device_id'=>$device_id,
            'rawdata'=>$rawdata,
            'appid'=>$this->appid,
            'mch_id'=>$this->mch_id,
            'now'=>time(),
            'version'=>'1',
            'nonce_str'=>$this->getNonceStr(16),
            'sign_type'=>'MD5',
        ];
        if (!empty($this->sub_mch_id)){
            $params['sub_mch_id']=$this->sub_mch_id;
        }

        $params['sign']=$this->getSign($params,$this->key);

        $this->getHttp()->setUrl($url);
        $xml=$this->getHttp()->post($this->toXml($params));
        $result = json_decode(json_encode(simplexml_load_string($xml, 'SimpleXMLElement', LIBXML_NOCDATA)), true);

        if ($result['return_code']!='SUCCESS'){
            throw new \Exception($result['return_msg']);
        }

        return $result;

    }

    /**
     * 调起刷码支付
     * @author heyu 2020-01-19 11:07
     * @param string $no
     * @param int $total_fee
     * @param string $auth_code
     * @param string $body
     * @return array
     * @throws \Exception
     */
    public function microPay($no,$total_fee,$auth_code,$body){
        $url='https://api.mch.weixin.qq.com/pay/micropay';
        $params=[
            'appid'=>$this->appid,
            'mch_id'=>$this->mch_id,
            'nonce_str'=>$this->getNonceStr(16),
            'sign_type'=>'MD5',
            'body'=>$body,
            'out_trade_no'=>$no,
            'total_fee'=>$total_fee,
            'spbill_create_ip'=>'115.29.165.48',
            'auth_code'=>$auth_code,
        ];
        if (!empty($this->sub_mch_id)){
            $params['sub_mch_id']=$this->sub_mch_id;
        }

        $params['sign']=$this->getSign($params,$this->key);

        $this->getHttp()->setUrl($url);
        $xml=$this->getHttp()->post($this->toXml($params));
        return json_decode(json_encode(simplexml_load_string($xml, 'SimpleXMLElement', LIBXML_NOCDATA)), true);
    }

    /**
     * 查询交易结果
     * @author heyu 2020-01-19 11:08
     * @param string $no
     * @return array
     * @throws \Exception
     */
    public function query($no){
        $url='https://api.mch.weixin.qq.com/pay/orderquery';
        $params=[
            'appid'=>$this->appid,
            'mch_id'=>$this->mch_id,
            'nonce_str'=>$this->getNonceStr(16),
            'sign_type'=>'MD5',
            'out_trade_no'=>$no,
        ];
        if (!empty($this->sub_mch_id)){
            $params['sub_mch_id']=$this->sub_mch_id;
        }

        $params['sign']=$this->getSign($params,$this->key);

        $this->getHttp()->setUrl($url);
        $xml=$this->getHttp()->post($this->toXml($params));

        return json_decode(json_encode(simplexml_load_string($xml, 'SimpleXMLElement', LIBXML_NOCDATA)), true);
    }

    /**
     * 发起退款
     * @author heyu 2020-01-19 11:10
     * @param string $no
     * @return mixed
     * @throws \Exception
     */
    public function refund($no){
        $url='https://api.mch.weixin.qq.com/pay/orderquery';
        $params=[
            'appid'=>$this->appid,
            'mch_id'=>$this->mch_id,
            'nonce_str'=>$this->getNonceStr(16),
            'sign_type'=>'MD5',
            'out_trade_no'=>$no,
        ];
        if (!empty($this->sub_mch_id)){
            $params['sub_mch_id']=$this->sub_mch_id;
        }

        $params['sign']=$this->getSign($params,$this->key);

        $this->getHttp()->setUrl($url);
        $xml=$this->getHttp()->post($this->toXml($params));

        return json_decode(json_encode(simplexml_load_string($xml, 'SimpleXMLElement', LIBXML_NOCDATA)), true);
    }
}