<?php
/**
 *
 * Auther: heyu
 * DateTime: 2019-11-01 10:59
 *
 */

namespace kwy\wechatapi\wxcard;

use kwy\util\http\CurlHttp;
use kwy\wechatapi\wxcard\entity\WxCardUpdateEntity;
use kwy\wechatapi\wxcard\entity\WxCardEntity;
use kwy\wechatapi\wxcard\entity\WxMemberCardEntity;

class WxCard
{

    /**
     * @var CurlHttp
     */
    private $httpClient;

    private $accessToken;

    /**
     * @author heyu 2019-11-01 16:41
     * @param WxCardEntity $wxCardEntity
     * @return array
     * @throws \Exception
     */
    public function create(WxCardEntity $wxCardEntity){
        $url='http://api.weixin.qq.com/card/create?access_token=ACCESS_TOKEN';
        $ret = $this->getHttpClient()
            ->setUrl($this->getUrlWithAccessToken($url))
            ->post(json_encode($wxCardEntity->getData(),JSON_UNESCAPED_UNICODE));
        return $this->resolveReturn($ret);
    }

    /**
     * 更新微信卡包信息
     * @author roc
     * @param string $card_id 微信会员卡id
     * @param WxCardEntity | WxCardUpdateEntity $wxCardEntity
     * @return array
     * @throws \Exception
     */
    public function update(string $card_id, WxCardUpdateEntity $wxCardEntity){
        $url = 'https://api.weixin.qq.com/card/update?access_token=ACCESS_TOKEN';
        $data = $wxCardEntity->getData();
        $data['card_id'] = $card_id;

        $ret = $this->getHttpClient()
            ->setUrl($this->getUrlWithAccessToken($url))
            ->post(json_encode($data,JSON_UNESCAPED_UNICODE));
        return $this->resolveReturn($ret);
    }

    /**
     * 设置会员卡开卡字段  用于一键开卡模式，该模式必须调用该方法设置开卡字段 否则一键开卡设置不生效
     * @author heyu 2019-11-04 17:58
     * @param string $card_id
     * @param WxMemberCardEntity $wxMemberCardEntity
     * @return array
     * @throws \Exception
     */
    public function memberCardWxActivate($card_id,WxMemberCardEntity $wxMemberCardEntity){
        $url='https://api.weixin.qq.com/card/membercard/activateuserform/set?access_token=ACCESS_TOKEN';
        $data=$wxMemberCardEntity->getActivateData();
        $data['card_id']=$card_id;
        return $this->resolveReturn($this->getHttpClient()
            ->setUrl($this->getUrlWithAccessToken($url))
            ->post(json_encode($data,JSON_UNESCAPED_UNICODE)));
    }

    /**
     * 跳转式一键开卡流程  根据激活ticket 获取当时用户填写的会员信息
     * @author heyu 2019-11-05 09:50
     * @param string $activate_ticket 激活ticket
     * @return array
     * @throws \Exception
     */
    public function getMemberInfoByActivateTicket($activate_ticket){
        $url='https://api.weixin.qq.com/card/membercard/activatetempinfo/get?access_token=ACCESS_TOKEN';
        $data=['activate_ticket'=>$activate_ticket];
        return $this->resolveReturn($this->getHttpClient()
            ->setUrl($this->getUrlWithAccessToken($url))
            ->post(json_encode($data,JSON_UNESCAPED_UNICODE)));
    }

    /**
     * 激活会员卡
     * @author heyu 2019-11-05 10:08
     * @param string $membership_number
     * @param string $code
     * @param array $member_info  选填信息
     * "init_bonus" 初始积分，不填为0
     * "init_bonus_record" 积分同步说明
     * "init_balance" 初始余额，不填为0
     * "init_custom_field_value1" 创建时字段custom_field1定义类型的初始值，限制为4个汉字，12字节
     * "init_custom_field_value2" 创建时字段custom_field2定义类型的初始值，限制为4个汉字，12字节
     * "init_custom_field_value3" 创建时字段custom_field3定义类型的初始值，限制为4个汉字，12字节
     * "card_id"卡券ID,自定义code卡券必填
     * "background_pic_url" 商家自定义会员卡背景图，须先调用 上传图片接口 将背景图上传至CDN，否则报错， 卡面设计请遵循 微信会员卡自定义背景设计规范
     * "activate_begin_time" 激活后的有效起始时间。若不填写默认以创建时的 data_info 为准。Unix时间戳格式
     * "activate_end_time" 激活后的有效截至时间。若不填写默认以创建时的 data_info 为准。Unix时间戳格式
     * @return array
     * @throws \Exception
     */
    public function activateMemberCard($membership_number,$code,$member_info=[]){
        $url='https://api.weixin.qq.com/card/membercard/activate?access_token=ACCESS_TOKEN';
        $data=[
            'membership_number'=>$membership_number,
            'code'=>$code,
        ];
        $data=array_merge($data,$member_info);
        return $this->resolveReturn($this->getHttpClient()
            ->setUrl($this->getUrlWithAccessToken($url))
            ->post(json_encode($data,JSON_UNESCAPED_UNICODE)));
    }

    /**
     * 同步会员数据 信息  当会员持卡消费后，支持开发者调用该接口更新会员信息。会员卡交易后的每次信息变更需通过该接口通知微信，便于后续消息通知及其他扩展功能
     * @author heyu 2019-11-05 10:16
     * @param string $code
     * @param string $card_id
     * @param array $member_info 选填会员信息
     * "balance" （int）需要设置的余额全量值，传入的数值会直接显示在卡面
     * "add_balance" （int） 本次余额变动值，传负数代表减少
     * "record_balance" 商家自定义金额消耗记录，不超过14个汉字
     * "bonus" （int） 需要设置的积分全量值，传入的数值会直接显示
     * "add_bonus" 本次积分变动值，传负数代表减少
     * "record_bonus" 商家自定义积分消耗记录，不超过14个汉字
     * "custom_field_value1" 创建时字段custom_field1定义类型的最新数值，限制为4个汉字，12字节，如"白金"
     * "custom_field_value2" 创建时字段custom_field2定义类型的最新数值，限制为4个汉字，12字节，如"8折"
     * "custom_field_value3" 创建时字段custom_field3定义类型的最新数值，限制为4个汉字，12字节，如"500"
     * @return array
     * @throws \Exception
     */
    public function updateMemberInfo($code,$card_id,$member_info=[]){
        $url='https://api.weixin.qq.com/card/membercard/updateuser?access_token=ACCESS_TOKEN';
        $data=[
            'code'=>$code,
            'card_id'=>$card_id,
            'notify_optional'=>[
                'is_notify_bonus'=>true,
                'is_notify_balance'=>true,
                'is_notify_custom_field1'=>true,
            ],
        ];
        $data=array_merge($data,$member_info);
        return $this->resolveReturn($this->getHttpClient()
            ->setUrl($this->getUrlWithAccessToken($url))
            ->post(json_encode($data,JSON_UNESCAPED_UNICODE)));
    }

    /**
     * 通过二维码发放卡券
     * @author heyu 2019-11-01 19:29
     * @param string $card_id 被方法的卡券id
     * @param string $code 卡券code 选填
     * @param string $openid 接收用户的openid
     * @param bool $is_unique_code 指定下发二维码，生成的二维码随机分配一个code，领取后不可再次扫描。填写true或false。默认false
     * @param int $expire 二维码过期时间 秒
     * @param string $outer_id 领取场景值，用于领取渠道的数据统计，默认值为0，字段类型为整型，长度限制为60位数字。用户领取卡券后触发的 事件推送 中会带上此自定义场景值
     * @param string $outer_str outer_id字段升级版本，字符串类型，用户首次领卡时，会通过 领取事件推送 给商户； 对于会员卡的二维码，用户每次扫码打开会员卡后点击任何url，会将该值拼入url中，方便开发者定位扫码来源
     * @return array
     * @example {"errcode": 0,"errmsg": "ok","ticket":  "gQHB8DoAAAAAAAAAASxodHRwOi8vd2VpeGluLnFxLmNvbS9xL0JIV3lhX3psZmlvSDZmWGVMMTZvAAIEsNnKVQMEIAMAAA==",  //获取ticket后需调用换取二维码接口获取二维码图片，详情见字段说明。"expire_seconds": 1800,"url": "http://weixin.qq.com/q/BHWya_zlfioH6fXeL16o ","show_qrcode_url": " https://mp.weixin.qq.com/cgi-bin/showqrcode?  ticket=gQH98DoAAAAAAAAAASxodHRwOi8vd2VpeGluLnFxLmNvbS9xL0czVzRlSWpsamlyM2plWTNKVktvAAIE6SfgVQMEgDPhAQ%3D%3D"}
     * @throws \Exception
     */
    public function generateWxCardQrCode($card_id,$code='',$openid='',$is_unique_code=false,$expire=1800,$outer_id='',$outer_str=''){
        $url='https://api.weixin.qq.com/card/qrcode/create?access_token=ACCESS_TOKEN';

        $data=[
            'action_name'=>'QR_CARD',
            'expire_seconds'=>$expire,
            'action_info'=>[
                'card'=>[
                    'card_id'=>$card_id,
                    'code'=>$code,
                    'openid'=>$openid,
                    'is_unique_code'=>$is_unique_code,
                    'outer_id'=>$outer_id,
                    'outer_str'=>$outer_str,
                ],
            ],
        ];

        $ret = $this->getHttpClient()
            ->setUrl($this->getUrlWithAccessToken($url))
            ->post(json_encode($data,JSON_UNESCAPED_UNICODE));
        return $this->resolveReturn($ret);
    }

    /**
     * @author heyu 2019-11-04 15:55
     * @param array $openidList 微信用户openid数组  可选
     * @param array $usernameList 微信用户账号数组 可选
     * @return array
     * @throws \Exception
     */
    public function setTestWhiteList($openidList,$usernameList){
        $url='https://api.weixin.qq.com/card/testwhitelist/set?access_token=ACCESS_TOKEN';
        $data=[
            'openid'=>$openidList,
            'username'=>$usernameList,
        ];
        return $this->resolveReturn($this->getHttpClient()->setUrl($this->getUrlWithAccessToken($url))->post(json_encode($data,JSON_UNESCAPED_UNICODE)));

    }


    public function setAccessToken($accessToken){
        $this->accessToken=$accessToken;
    }

    /**
     * @author heyu 2019-11-01 19:13
     * @param string $arr
     * @return array
     * @throws \Exception
     */
    protected function resolveReturn($str){
        $arr=json_decode($str,true);
        if ($arr['errcode']!==0){
            $errmsg='';
            switch ($arr['errcode']){
                case 40070:$errmsg='基本信息base info中填写的库存信息SKU不合法，参考CreateCard创建卡券接口';break;
                case 40104:$errmsg='创建子商户时填的类目id不对';break;
                case 40139:$errmsg='子商户id不正确';break;
                case 40140:$errmsg='子商户状态不对，创建卡券时要求已审核，更新子商户信息时要求已驳回';break;
                case 42001:$errmsg='授权码ACCESS_TOKEN已过期';break;
                default:$errmsg=$arr['errmsg'].',code='.$arr['errcode'];
            }
            throw new \Exception($errmsg);
        }
        return $arr;
    }

    private function getUrlWithAccessToken($url){
        if (empty($this->accessToken)){
            throw new \Exception('accessToken 未设置');
        }
        return str_replace('ACCESS_TOKEN',$this->accessToken,$url);
    }

    public function getHttpClient(){
        if (empty($this->httpClient)){
            $this->httpClient = new CurlHttp();
            //$this->httpClient->setProxy('127.0.0.1:8888');
            $this->httpClient->setContentType('application/json');
        }
        return $this->httpClient;
    }
}